/* 
 * SDLInvasion a Space Invader clone.
 * Copyright (C) 2002 Laurent GOMEZ & MiKael NAVARRO
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */


#include <stdio.h>
#include <stdlib.h>

#include "getopt.h" /* this file is now part of the GNU C Library */

#include "SDL.h"

#include "invasion.h"

#include "player.h"


/* Print help/usage 
 */
static int print_usage(void) {
  printf("Usage: SDLInvasion [--help]\n");
  printf("SDLInvasion a Space Invader clone.\n\n");
  printf("  Options:\n");
  printf("  --help         display this message and exit\n");
  printf("\nReport bugs to mnavarro@caramail.com or iori@ifrance.com\n");
  return 0;
}

/* Check video modes
 */
static int checkVideoModes() {
  SDL_VideoInfo *video_info;
  SDL_Rect **video_modes;
  SDL_PixelFormat *best_vfmt;
  char driver_name[8];
  Uint32 bpp;
  int i;
 
  /* Get some infos on system */
  if(SDL_VideoDriverName(driver_name, 8)) {
     fprintf(stdout, "VideoDriverName: %s\n", driver_name);
  }

  video_info = (SDL_VideoInfo *)SDL_GetVideoInfo();
  if(video_info) {
    fprintf(stdout, "  hardware surfaces? %d\n", video_info->hw_available);
    fprintf(stdout, "  window manager? %d\n", video_info->wm_available);
    fprintf(stdout, "  hardware accel? %d\n", video_info->blit_hw);
    fprintf(stdout, "  video memory: %u Kb\n", video_info->video_mem);
    best_vfmt = video_info->vfmt;
    fprintf(stdout, "'best' available mode: %dbpp\n", best_vfmt->BitsPerPixel);
  }

  /* Get available software/hardware/fullscreen modes */
  video_modes = SDL_ListModes(NULL, 
			      SDL_HWSURFACE | 
			      SDL_SWSURFACE | 
			      SDL_FULLSCREEN | 
			      SDL_ANYFORMAT);
  /* Check is there are any modes available */
  if(video_modes == (SDL_Rect **)0) {
    fprintf(stdout, "No video modes available!\n");
    exit(1);
  }
  /* Check if or resolution is restricted */
  if(video_modes == (SDL_Rect **)-1) {
    fprintf(stdout, "All resolutions available.\n");
  }
  else { /* print valid modes */
    fprintf(stdout, "Available video modes:\n");
    for(i=0; video_modes[i]; ++i)
      fprintf(stdout, "  %d x %d\n", video_modes[i]->w, video_modes[i]->h);
  }

  /* Check video modes */
  fprintf(stdout, "Hadware: checking mode 640 x 480 @ 16bpp...\n");
  bpp = SDL_VideoModeOK(640, 480, 16, SDL_HWSURFACE);
  if(!bpp) {
     printf("Mode not available!\n");
     exit(1);
  }
  fprintf(stdout, "  SDL recommends 640 x 480 @ %dbpp.\n", bpp);

  fprintf(stdout, "Software: checking mode 640 x 480 @ 16bpp...\n");
  bpp = SDL_VideoModeOK(640, 480, 16, SDL_SWSURFACE);
  if(!bpp) {
     printf("Mode not available!\n");
     exit(1);
  }
  fprintf(stdout, "  SDL recommends 640 x 480 @ %dbpp.\n", bpp);

  fprintf(stdout, "Fullscreen: checking mode 640 x 480 @ 16bpp...\n");
  bpp = SDL_VideoModeOK(640, 480, 16, SDL_FULLSCREEN);
  if(!bpp) {
     printf("Mode not available!\n");
     exit(1);
  }
  fprintf(stdout, "  SDL recommends 640 x 480 @ %dbpp.\n", bpp);

  fprintf(stdout, "\n");
  return 0;
}


/* Main program 
 */
int main(int argc, char *argv[]) {
  int c, opt_ind=0;
  static struct option long_options[] = {
    {"player_sprite", 1, 0, 'p'},
    {"help", 0, 0, 'h'},
    {0, 0, 0, 0}
  };
  char player_sprite[256]="data/alien.bmp"; /* default player sprite */
  SDL_Surface *screen; /* screen */
  Player *player = NULL;

  fprintf(stdout, "\n");
  fprintf(stdout, "SDLInvasion a Space Invader clone.\n");
  fprintf(stdout, "\n");
  fprintf(stdout, "Copyright (C) 2002 Laurent GOMEZ & MiKael NAVARRO\n");
  fprintf(stdout, "SDLInvasion is distributed in the hope that it will be fun,\n");
  fprintf(stdout, "but WITHOUT ANY WARRANTY.\n");
  fprintf(stdout, "\n");

  /* Options filtering */
  while((c=getopt_long_only(argc, argv, "hp:", long_options, &opt_ind))!=EOF) {
    switch(c) {
    case 'p':
      strcpy(player_sprite, optarg); break;
    case 'h':
      print_usage(); return 0;
    default:
      break;
    }
  }
  /* Other options */
  if(optind < argc) {
  }
  else {
    /* Too few arguments */
  }

  /* Initialize the SDL library */
  if(SDL_Init(SDL_INIT_VIDEO) < 0) {
    fprintf(stderr, "Couldn't initialize SDL: %s\n", SDL_GetError());
    exit(1);
  }
  atexit(SDL_Quit);
  
  /* Check some video modes */
  checkVideoModes();

  /* Initialize video, have a preference for 16-bit, but accept any depth */
  screen = SDL_SetVideoMode(640, 480, 16, SDL_HWSURFACE | SDL_ANYFORMAT);
  if(!screen) {
    fprintf(stderr, "Couldn't initialize 640 x 480 @ 16bpp: %s\n", 
            SDL_GetError());
    exit(1);
  }
  fprintf(stdout, "Set 640 x 480 @ %d bpp mode\n", screen->format->BitsPerPixel);
  SDL_WM_SetCaption("SDLInvasion", "SDLInvasion");
  
  /* Init player */
  player = Player_new(screen, player_sprite);

  /* Run Invasion ! */
  runInvasion(screen, player);

  /* Release memory */
  Player_destroy(player);

  return 0; /* ANSI C requires main to return int */
}
